/*
 * Parameters.java
 *
 * Created on 27 novembre 2004, 22:40
 */

package ittp;

import ittp.protocol.request.Request;
import ittp.protocol.response.Response;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.HashMap;

/**
 * This class allows to define the different using's parameters of the server.
 * It use RMI.
 * @author Frederic Bidon and Mathieu Texier
 */
public class Parameters extends UnicastRemoteObject implements RemoteControl {
    
    /**
     * Number maximal of connection allowed
     */
    static final public int CONNECTION_MAX = 100;
    
    /**
     * timout maximal allowed
     */
    static final public int TIMEOUT_MAX = 100;
    
    /**
     * default value for the number of connection
     */
    static final public int DEFAULT_CONNECTION = 10;
    
    /**
     * default value for the time out
     */
    static final public int DEFAULT_TIMEOUT = 10;
    
    /**
     * start state
     */
    static final public String START = "started";
    
    /**
     * stop state
     */
    static final public String STOP = "stoped";
    
    /**
     * shut down state
     */
    static final public String SHUT_DOWN = "Shuted Down";
    
    private boolean[] allowedMethod = new boolean[] {true, true, true, true};
    private HashMap allowedHeader = new HashMap ();
    
    /**
     * int port defines the number of the port used for the connection between the server and the client
     */
    private int port = 36000;
    
    /**
     * String baseDir defines the directory of the ressource.
     */
    private String baseDir = System.getProperty ("user.dir");
    
    /**
     * int countRequest defines the number of request send to the server.
     */
    private int countRequest = 0;
    
    /**
     * int countError defines the number of error returned by the server.
     */
    private int countError = 0;
    
    /**
     * int countOctet defines the number of octet transferred by the server.
     */
    private int countOctet = 0;
    
    /**
     * int countConnect defines the number of connection etablished with the server.
     */
    private int countConnect = 0;
    
    /**
     * int timeOut defines the number of the time inactivaty authorized.
     */
    private int timeOut = 10;
    
    /**
     * int connectionMax defines the number max of connections authorized in the same time.
     */
    private int connectionMax = 10;
    
    /**
     * int serverState defines the state of the server.
     */
    private String serverState = START;
    
    /**
     * Constructor of this class
     * @throws RemoteException when the ServerSocket cannot be opened.
     */
    public Parameters () throws RemoteException {
        super ();
        allowedHeader.put (Response.ALLOW, new Boolean (true));
        allowedHeader.put (Response.CONTENT_MD5, new Boolean (true));
        allowedHeader.put (Response.CONTENT_TYPE, new Boolean (true));
        allowedHeader.put (Response.SERVER, new Boolean (true));
    }
    
    
    // All the methods that follow allow to access or modify the previous variable.    
    
    synchronized public boolean isAllowedMethod (int request) throws RemoteException {
        return allowedMethod[request];
    }
    
    synchronized public boolean isAllowedHeader (String id) throws RemoteException {
        return ((Boolean) allowedHeader.get (id)).booleanValue ();
    }
    
    synchronized public String getServerState () throws RemoteException {
        return serverState;
    }
    
    synchronized public void setPort (int p) throws RemoteException {
        port = p;
    }
    
    synchronized public void setBaseDir (String baseDir) throws RemoteException {
        this.baseDir = baseDir;
    }
    
    synchronized public int getPort () throws RemoteException {
        return port;
    }
    
    synchronized public String getBaseDir () throws RemoteException {
        if (!baseDir.endsWith ("\\") || !baseDir.endsWith ("/"))
            return baseDir  + "/";
        else
            return baseDir;
    }
    
    synchronized public int getNumberOfRequest () throws RemoteException {
        return countRequest;
    }
    
    synchronized public int getNumberOfError () throws RemoteException {
        return countError;
    }
    
    synchronized public int getNumberOfOctet () throws RemoteException {
        return countOctet;
    }
    
    synchronized public int getNumberOfConnect () throws RemoteException {
        return countConnect;
    }
    
    synchronized public int getTimeOut () throws RemoteException {
        return timeOut;
    }
    
    synchronized public void setAllowedMethod (int request, boolean value) throws RemoteException {
        allowedMethod[request] = value;
    }
    
    synchronized public void setAllowedHeader (String id, boolean value) throws RemoteException {
        allowedHeader.put (id, new Boolean (value));
    }
    
    synchronized public int getConnectionMax () throws RemoteException {
        return connectionMax;
    }
    
    synchronized public int setTimeOut (int timeOut) throws RemoteException {
        if (timeOut < 0)
            timeOut = 0;
        if (timeOut > TIMEOUT_MAX)
            timeOut = TIMEOUT_MAX;
        this.timeOut = timeOut;
        return timeOut;
    }
    
    synchronized public int setConnectionMax (int connectionMax) throws RemoteException {
        if (connectionMax < 0)
            connectionMax = 0;
        if (connectionMax > CONNECTION_MAX)
            connectionMax = CONNECTION_MAX;
        this.connectionMax = connectionMax;
        return connectionMax;
    }
    
    synchronized public void setServerState (String state) throws RemoteException {
        serverState = state;
    }
    
    synchronized public int getNumberOfActiveConection () throws RemoteException {
        return Server.getInstance ().threadGroup.activeCount ();
    }
    
    synchronized public void recordError (){
        countError ++;
    }
    
    synchronized public void recordRequest (){
        countRequest ++;
    }
    
    synchronized public void recordOctet (long octet){
        countOctet += octet;
    }
    
    synchronized public void recordConnect (){
        countConnect ++;
    }
    
    synchronized public void resetRecord () throws RemoteException {
        countError = 0;
        countRequest = 0;
        countOctet = 0;
        countConnect = 0;
    }
}
